@@ -4,13 +4,13 @@ from types import *
 from itertools import ifilterfalse
 import re
 import functools
+import random
 from sets import Set
 from threading import Timer
 
 
-class _IdCounter:
-    """
-    A Global Dictionary for uniq IDs
+class _IdCounter(object):
+    """ A Global Dictionary for uniq IDs
     """
     count = 0
     pass
@@ -35,8 +35,7 @@ class __(object):
 
 
 def u_withrepr(reprfun):
-    """
-    Decorator to rename a function
+    """ Decorator to rename a function
     """
     def _wrap(func):
         return __(reprfun, func)
@@ -53,7 +52,7 @@ def _(obj):
     return underscore(obj)
 
 
-class underscore():
+class underscore(object):
     """
     Instead of creating a class named _ (underscore) I created underscore
     So I can use _ function both statically and dynamically just it
@@ -61,42 +60,41 @@ class underscore():
     """
 
     object = None
-    """
-    Passed object
+    """ Passed object
     """
 
-    VERSION = "0.1.2"
+    VERSION = "0.1.3"
 
     chained = False
-    """
-    If the object is in a chained state or not
+    """ If the object is in a chained state or not
     """
 
     Null = "__Null__"
     """
     Since we are working with the native types
-    I cannot compare anything with None, so I use a Substitute type for checking
+    I can't compare anything with None, so I use a Substitute type for checking
     """
 
     _wrapped = Null
     """
-    When object is in chained state, This property will contain the latest processed
-    Value of passed object, I assign it no Null so I can check against None results
+    When object is in chained state, This property will contain the latest
+    processed Value of passed object, I assign it no Null so I can check
+    against None results
     """
 
-    class Namespace:
-        """
-        For simulating full closure support
-        """
-        pass
-
     def __init__(self, obj):
-        """
-        Let there be light
+        """ Let there be light
         """
         self.chained = False
         self.object = obj
 
+        class Namespace(object):
+            """ For simulating full closure support
+            """
+            pass
+
+        self.Namespace = Namespace
+
     def __str__(self):
         if self.chained is True:
             return "Underscore chained instance"
@@ -120,6 +118,13 @@ class underscore():
         else:
             return self.object
 
+    @obj.setter
+    def obj(self, value):
+        """ New style classes requires setters for @propert methods
+        """
+        self.object = value
+        return self.object
+
     def _wrap(self, ret):
         """
         Returns result but ig chain method is used
@@ -140,8 +145,7 @@ class underscore():
         return _(self.obj)
 
     def _toOriginal(self, val):
-        """
-        Pitty attempt to convert itertools result into a real object
+        """ Pitty attempt to convert itertools result into a real object
         """
         if self._clean.isTuple():
             return tuple(val)
@@ -175,8 +179,7 @@ class underscore():
     forEach = each
 
     def map(self, func):
-        """
-        Return the results of applying the iterator to each element.
+        """ Return the results of applying the iterator to each element.
         """
         ns = self.Namespace()
         ns.results = []
@@ -190,7 +193,8 @@ class underscore():
 
     def reduce(self, func, memo=None):
         """
-        **Reduce** builds up a single result from a list of values, aka `inject`, or foldl
+        **Reduce** builds up a single result from a list of values,
+        aka `inject`, or foldl
         """
         if memo is None:
             memo = []
@@ -211,8 +215,7 @@ class underscore():
     foldl = inject = reduce
 
     def reduceRight(self, func):
-        """
-        The right-associative version of reduce, also known as `foldr`.
+        """ The right-associative version of reduce, also known as `foldr`.
         """
         #foldr = lambda f, i: lambda s: reduce(f, s, i)
         x = self.obj[:]
@@ -222,7 +225,8 @@ class underscore():
 
     def find(self, func):
         """
-        Return the first value which passes a truth test. Aliased as `detect`.
+        Return the first value which passes a truth test.
+        Aliased as `detect`.
         """
         self.ftmp = None
 
@@ -235,22 +239,19 @@ class underscore():
     detect = find
 
     def filter(self, func):
-        """
-        Return all the elements that pass a truth test.
+        """ Return all the elements that pass a truth test.
         """
         return self._wrap(filter(func, self.obj))
     select = filter
 
     def reject(self, func):
-        """
-        Return all the elements for which a truth test fails.
+        """ Return all the elements for which a truth test fails.
         """
         r = ifilterfalse(func, self.obj)
         return self._wrap(self._toOriginal(r))
 
     def all(self, func=None):
-        """
-        Determine whether all of the elements match a truth test.
+        """ Determine whether all of the elements match a truth test.
         """
         if func is None:
             func = lambda x, *args: x
@@ -266,7 +267,8 @@ class underscore():
 
     def any(self, func=None):
         """
-        Determine if at least one element in the object matches a truth test.
+        Determine if at least one element in the object
+        matches a truth test.
         """
         if func is None:
             func = lambda x, *args: x
@@ -283,7 +285,8 @@ class underscore():
 
     def include(self, target):
         """
-        Determine if a given value is included in the array or object using `is`.
+        Determine if a given value is included in the
+        array or object using `is`.
         """
         if self._clean.isDict():
             return self._wrap(target in self.obj.values())
@@ -292,11 +295,14 @@ class underscore():
     contains = include
 
     def invoke(self, method, *args):
-        """
-        Invoke a method (with arguments) on every item in a collection.
+        """ Invoke a method (with arguments) on every item in a collection.
         """
         def inv(value, *ar):
-            if(_(method).isFunction() or _(method).isLambda() or _(method).isMethod()):
+            if (
+                _(method).isFunction() or
+                _(method).isLambda() or
+                _(method).isMethod()
+            ):
                 return method(value, *args)
             else:
                 return getattr(value, method)(*args)
@@ -304,59 +310,83 @@ class underscore():
 
     def pluck(self, key):
         """
-        Convenience version of a common use case of `map`: fetching a property.
+        Convenience version of a common use case of
+        `map`: fetching a property.
         """
         return self._wrap([x.get(key) for x in self.obj])
 
-    def max(self):
+    def where(self, attrs=None, first=False):
         """
-        Return the maximum element or (element-based computation).
+        Convenience version of a common use case of `filter`: selecting only objects
+        containing specific `key:value` pairs.
+        """
+        if attrs is None:
+            return None if first is True else []
+
+        method = _.find if first else _.filter
+
+        def by(val, *args):
+            for key, value in attrs.items():
+                try:
+                    if attrs[key] != val[key]:
+                        return False
+                except KeyError:
+                    return False
+                return True
+
+        return self._wrap(method(self.obj, by))
+
+    def findWhere(self, attrs=None):
+        """
+        Convenience version of a common use case of `find`: getting the first object
+        containing specific `key:value` pairs.
+        """
+        return self._wrap(self._clean.where(attrs, True))
+
+    def max(self):
+        """ Return the maximum element or (element-based computation).
         """
         if(self._clean.isDict()):
             return self._wrap(list())
         return self._wrap(max(self.obj))
 
     def min(self):
-        """
-        Return the minimum element (or element-based computation).
+        """ Return the minimum element (or element-based computation).
         """
         if(self._clean.isDict()):
             return self._wrap(list())
         return self._wrap(min(self.obj))
 
     def shuffle(self):
-        """
-        Shuffle an array.
+        """ Shuffle an array.
         """
         if(self._clean.isDict()):
             return self._wrap(list())
 
         cloned = self.obj[:]
-        import random
+
         random.shuffle(cloned)
         return self._wrap(cloned)
 
     def sortBy(self, val=None):
-        """
-        Sort the object's values by a criterion produced by an iterator.
+        """ Sort the object's values by a criterion produced by an iterator.
         """
         if val is not None:
             if _(val).isString():
-                return self._wrap(sorted(self.obj, key=lambda x, *args: x.get(val)))
+                return self._wrap(sorted(self.obj, key=lambda x,
+                                  *args: x.get(val)))
             else:
                 return self._wrap(sorted(self.obj, val))
         else:
             return self._wrap(sorted(self.obj))
 
     def _lookupIterator(self, obj, val):
-        """
-        An internal function to generate lookup iterators.
+        """ An internal function to generate lookup iterators.
         """
         return val if _.isCallable(val) else lambda obj, *args: obj[val]
 
     def _group(self, obj, val, behavior):
-        """
-        An internal function used for aggregate "group by" operations.
+        """ An internal function used for aggregate "group by" operations.
         """
         ns = self.Namespace()
         ns.result = {}
@@ -368,12 +398,14 @@ class underscore():
 
         _.each(obj, e)
 
+        if len(ns.result) == 1:
+            return ns.result[0]
         return ns.result
 
     def groupBy(self, val):
         """
-        Groups the object's values by a criterion. Pass either a string attribute
-        to group by, or a function that returns the criterion.
+        Groups the object's values by a criterion. Pass either a string
+        attribute to group by, or a function that returns the criterion.
         """
 
         def by(result, key, value):
@@ -385,6 +417,21 @@ class underscore():
 
         return self._wrap(res)
 
+    def indexBy(self, val=None):
+        """
+        Indexes the object's values by a criterion, similar to `groupBy`, but for
+        when you know that your index values will be unique.
+        """
+        if val is None:
+            val = lambda *args: args[0]
+
+        def by(result, key, value):
+            result[key] = value
+
+        res = self._group(self.obj, val, by)
+
+        return self._wrap(res)
+
     def countBy(self, val):
         """
         Counts instances of an object that group by a certain criterion. Pass
@@ -404,7 +451,8 @@ class underscore():
     def sortedIndex(self, obj, iterator=lambda x: x):
         """
         Use a comparator function to figure out the smallest index at which
-        an object should be inserted so as to maintain order. Uses binary search.
+        an object should be inserted so as to maintain order.
+        Uses binary search.
         """
         array = self.obj
         value = iterator(obj)
@@ -419,22 +467,20 @@ class underscore():
         return self._wrap(low)
 
     def toArray(self):
-        """
-        Safely convert anything iterable into a real, live array.
+        """ Safely convert anything iterable into a real, live array.
         """
         return self._wrap(list(self.obj))
 
     def size(self):
-        """
-        Return the number of elements in an object.
+        """ Return the number of elements in an object.
         """
         return self._wrap(len(self.obj))
 
     def first(self, n=1):
         """
-        Get the first element of an array. Passing **n** will return the first N
-        values in the array. Aliased as `head` and `take`. The **guard** check
-        allows it to work with `_.map`.
+        Get the first element of an array. Passing **n** will return the
+        first N values in the array. Aliased as `head` and `take`.
+        The **guard** check allows it to work with `_.map`.
         """
         res = self.obj[0:n]
         if len(res) is 1:
@@ -444,17 +490,18 @@ class underscore():
 
     def initial(self, n=1):
         """
-        Returns everything but the last entry of the array. Especially useful on
-        the arguments object. Passing **n** will return all the values in
-        the array, excluding the last N. The **guard** check allows it to work with
-        `_.map`.
+        Returns everything but the last entry of the array.
+        Especially useful on the arguments object.
+        Passing **n** will return all the values in the array, excluding the
+        last N. The **guard** check allows it to work with `_.map`.
         """
         return self._wrap(self.obj[0:-n])
 
     def last(self, n=1):
         """
         Get the last element of an array. Passing **n** will return the last N
-        values in the array. The **guard** check allows it to work with `_.map`.
+        values in the array.
+        The **guard** check allows it to work with `_.map`.
         """
         res = self.obj[-n:]
         if len(res) is 1:
@@ -464,16 +511,16 @@ class underscore():
     def rest(self, n=1):
         """
         Returns everything but the first entry of the array. Aliased as `tail`.
-        Especially useful on the arguments object. Passing an **index** will return
-        the rest of the values in the array from that index onward. The **guard**
-        check allows it to work with `_.map`.
+        Especially useful on the arguments object.
+        Passing an **index** will return the rest of the values in the
+        array from that index onward.
+        The **guard** check allows it to work with `_.map`.
         """
         return self._wrap(self.obj[n:])
     tail = rest
 
     def compact(self):
-        """
-        Trim out all falsy values from an array.
+        """ Trim out all falsy values from an array.
         """
         return self._wrap(self._clean.filter(lambda x: x))
 
@@ -497,14 +544,14 @@ class underscore():
         return ns.output
 
     def flatten(self, shallow=None):
-        """
-        Return a completely flattened version of an array.
+        """ Return a completely flattened version of an array.
         """
         return self._wrap(self._flatten(self.obj, shallow))
 
     def without(self, *values):
         """
-        Return a version of the array that does not contain the specified value(s).
+        Return a version of the array that does not
+        contain the specified value(s).
         """
         if self._clean.isDict():
             newlist = {}
@@ -535,7 +582,8 @@ class underscore():
             initial = _(ns.array).map(iterator)
 
         def by(memo, value, index):
-            if (_.last(memo) != value or not len(memo)) if isSorted else not _.include(memo, value):
+            if ((_.last(memo) != value or
+                 not len(memo)) if isSorted else not _.include(memo, value)):
                 memo.append(value)
                 ns.results.append(ns.array[index])
 
@@ -551,8 +599,8 @@ class underscore():
 
     def union(self, *args):
         """
-        Produce an array that contains the union: each distinct element from all of
-        the passed-in arrays.
+        Produce an array that contains the union: each distinct element
+        from all of the passed-in arrays.
         """
         # setobj = Set(self.obj)
         # for i, v in enumerate(args):
@@ -601,8 +649,8 @@ class underscore():
 
     def zipObject(self, values):
         """
-        Zip together two arrays -- an array of keys and an array of values -- into
-        a single object.
+        Zip together two arrays -- an array of keys and an array
+        of values -- into a single object.
         """
         result = {}
         keys = self.obj
@@ -655,8 +703,7 @@ class underscore():
         return self._wrap(-1)
 
     def range(self, *args):
-        """
-        Generate an integer Array containing an arithmetic progression.
+        """ Generate an integer Array containing an arithmetic progression.
         """
         args = list(args)
         args.insert(0, self.obj)
@@ -668,22 +715,34 @@ class underscore():
 
     def bind(self, context):
         """
-        Create a function bound to a given object (assigning `this`, and arguments,
-        optionally). Binding with arguments is also known as `curry`.
+        Create a function bound to a given object (assigning `this`,
+        and arguments, optionally).
+        Binding with arguments is also known as `curry`.
         """
         return self._wrap(self.obj)
     curry = bind
 
+    def partial(self, *args):
+        """
+        Partially apply a function by creating a version that has had some of its
+        arguments pre-filled, without changing its dynamic `this` context.
+        """
+        def part(*args2):
+            args3 = args + args2
+            return self.obj(*args3)
+
+        return self._wrap(part)
+
     def bindAll(self, *args):
         """
-        Bind all of an object's methods to that object. Useful for ensuring that
-        all callbacks defined on an object belong to it.
+        Bind all of an object's methods to that object.
+        Useful for ensuring that all callbacks defined on an
+        object belong to it.
         """
         return self._wrap(self.obj)
 
     def memoize(self, hasher=None):
-        """
-        Memoize an expensive function by storing its results.
+        """ Memoize an expensive function by storing its results.
         """
         ns = self.Namespace()
         ns.memo = {}
@@ -713,17 +772,17 @@ class underscore():
 
     def defer(self, *args):
         """
-        Defers a function, scheduling it to run after the current call stack has
-        cleared.
+        Defers a function, scheduling it to run after
+        the current call stack has cleared.
         """
-        ## I know! this is not really a defer in python. I'm open to suggestions
-        #########################################################################
+        ## I know! this isn't really a defer in python. I'm open to suggestions
+        #######################################################################
         return self.delay(1, *args)
 
     def throttle(self, wait):
         """
-        Returns a function, that, when invoked, will only be triggered at most once
-        during a given window of time.
+        Returns a function, that, when invoked, will only be triggered
+        at most once during a given window of time.
         """
         ns = self.Namespace()
         ns.timeout = None
@@ -760,10 +819,10 @@ class underscore():
     # https://gist.github.com/2871026
     def debounce(self, wait, immediate=None):
         """
-        Returns a function, that, as long as it continues to be invoked, will not
-        be triggered. The function will be called after it stops being called for
-        N milliseconds. If `immediate` is passed, trigger the function on the
-        leading edge, instead of the trailing.
+        Returns a function, that, as long as it continues to be invoked,
+        will not be triggered. The function will be called after it stops
+        being called for N milliseconds. If `immediate` is passed, trigger
+        the function on the leading edge, instead of the trailing.
         """
         wait = (float(wait) / float(1000))
 
@@ -780,15 +839,15 @@ class underscore():
 
     def once(self):
         """
-        Returns a function that will be executed at most one time, no matter how
-        often you call it. Useful for lazy initialization.
+        Returns a function that will be executed at most one time,
+        no matter how often you call it. Useful for lazy initialization.
         """
         ns = self.Namespace()
         ns.memo = None
         ns.run = False
 
         def work_once(*args, **kwargs):
-            if ns.run == False:
+            if ns.run is False:
                 ns.memo = self.obj(*args, **kwargs)
             ns.run = True
             return ns.memo
@@ -831,7 +890,8 @@ class underscore():
 
     def after(self, func):
         """
-        Returns a function that will only be executed after being called N times.
+        Returns a function that will only be executed after being
+        called N times.
         """
         ns = self.Namespace()
         ns.times = self.obj
@@ -851,20 +911,37 @@ class underscore():
     """
 
     def keys(self):
-        """
-        Retrieve the names of an object's properties.
+        """ Retrieve the names of an object's properties.
         """
         return self._wrap(self.obj.keys())
 
     def values(self):
-        """
-        Retrieve the values of an object's properties.
+        """ Retrieve the values of an object's properties.
         """
         return self._wrap(self.obj.values())
 
-    def functions(self):
+    def pairs(self):
+        """ Convert an object into a list of `[key, value]` pairs.
         """
-        Return a sorted list of the function names available on the object.
+        keys = self._clean.keys()
+        pairs = []
+        for key in keys:
+            pairs.append([key, self.obj[key]])
+
+        return self._wrap(pairs)
+
+    def invert(self):
+        """ Invert the keys and values of an object. The values must be serializable.
+        """
+        keys = self._clean.keys()
+        inverted = {}
+        for key in keys:
+            inverted[self.obj[key]] = key
+
+        return self._wrap(inverted)
+
+    def functions(self):
+        """ Return a sorted list of the function names available on the object.
         """
         names = []
 
@@ -877,7 +954,8 @@ class underscore():
 
     def extend(self, *args):
         """
-        Extend a given object with all the properties in passed-in object(s).
+        Extend a given object with all the properties in
+        passed-in object(s).
         """
         args = list(args)
         for i in args:
@@ -887,7 +965,8 @@ class underscore():
 
     def pick(self, *args):
         """
-        Return a copy of the object only containing the whitelisted properties.
+        Return a copy of the object only containing the
+        whitelisted properties.
         """
         ns = self.Namespace()
         ns.result = {}
@@ -909,8 +988,7 @@ class underscore():
         return self._wrap(copy)
 
     def defaults(self, *args):
-        """
-        Fill in a given object with default properties.
+        """ Fill in a given object with default properties.
         """
         ns = self.Namespace
         ns.obj = self.obj
@@ -925,8 +1003,7 @@ class underscore():
         return self._wrap(ns.obj)
 
     def clone(self):
-        """
-        Create a (shallow-cloned) duplicate of an object.
+        """ Create a (shallow-cloned) duplicate of an object.
         """
         import copy
         return self._wrap(copy.copy(self.obj))
@@ -941,8 +1018,7 @@ class underscore():
         return self._wrap(self.obj)
 
     def isEqual(self, match):
-        """
-        Perform a deep comparison to check if two objects are equal.
+        """ Perform a deep comparison to check if two objects are equal.
         """
         return self._wrap(self.obj == match)
 
@@ -951,7 +1027,7 @@ class underscore():
         Is a given array, string, or object empty?
         An "empty" object has no enumerable own-properties.
         """
-        if self.obj == None:
+        if self.obj is None:
             return True
         if self._clean.isString():
             ret = self.obj.strip() is ""
@@ -962,250 +1038,211 @@ class underscore():
         return self._wrap(ret)
 
     def isElement(self):
-        """
-        No use in python
+        """ No use in python
         """
         return self._wrap(False)
 
     def isDict(self):
-        """
-        Check if given object is a dictionary DictType
+        """ Check if given object is a dictionary DictType
         """
         return self._wrap(type(self.obj) is DictType)
 
     def isTuple(self):
-        """
-        Check if given object is a Tuple TupleType
+        """ Check if given object is a Tuple TupleType
         """
         return self._wrap(type(self.obj) is TupleType)
 
     def isList(self):
-        """
-        Check if given object is a list ListType
+        """ Check if given object is a list ListType
         """
         return self._wrap(type(self.obj) is ListType)
 
     def isNone(self):
-        """
-        Check if the given object is NoneType
+        """ Check if the given object is NoneType
         """
         return self._wrap(type(self.obj) is NoneType)
 
     def isType(self):
-        """
-        Check if the given object is TypeType
+        """ Check if the given object is TypeType
         """
         return self._wrap(type(self.obj) is TypeType)
 
     def isBoolean(self):
-        """
-        Check if the given object is BooleanType
+        """ Check if the given object is BooleanType
         """
         return self._wrap(type(self.obj) is BooleanType)
     isBool = isBoolean
 
     def isInt(self):
-        """
-        Check if the given object is IntType
+        """ Check if the given object is IntType
         """
         return self._wrap(type(self.obj) is IntType)
 
     def isLong(self):
-        """
-        Check if the given object is LongType
+        """ Check if the given object is LongType
         """
         return self._wrap(type(self.obj) is LongType)
 
     def isFloat(self):
-        """
-        Check if the given object is FloatType
+        """ Check if the given object is FloatType
         """
         return self._wrap(type(self.obj) is FloatType)
 
     def isComplex(self):
-        """
-        Check if the given object is ComplexType
+        """ Check if the given object is ComplexType
         """
         return self._wrap(type(self.obj) is ComplexType)
 
     def isString(self):
-        """
-        Check if the given object is StringType
+        """ Check if the given object is StringType
         """
         return self._wrap(type(self.obj) is StringType)
 
     def isUnicode(self):
-        """
-        Check if the given object is UnicodeType
+        """ Check if the given object is UnicodeType
         """
         return self._wrap(type(self.obj) is UnicodeType)
 
     def isCallable(self):
+        """ Check if the given object is any of the function types
         """
-        Check if the given object is any of the function types
-        """
-        return self._wrap(type(self.obj) is MethodType \
-                       or type(self.obj) is FunctionType \
-                       or type(self.obj) is LambdaType \
-                       or type(self.obj) is BuiltinMethodType \
-                       or type(self.obj) is BuiltinFunctionType\
-                       or type(self.obj) is UnboundMethodType)
+        return self._wrap(type(self.obj) is MethodType
+                          or type(self.obj) is FunctionType
+                          or type(self.obj) is LambdaType
+                          or type(self.obj) is BuiltinMethodType
+                          or type(self.obj) is BuiltinFunctionType
+                          or type(self.obj) is UnboundMethodType)
 
     def isFunction(self):
-        """
-        Check if the given object is FunctionType
+        """ Check if the given object is FunctionType
         """
         return self._wrap(type(self.obj) is FunctionType)
 
     def isLambda(self):
+        """ Check if the given object is LambdaType
         """
-        Check if the given object is LambdaType
-        """
-        return self._wrap(type(self.obj) is LambdaType)  # or type(self.obj) is FunctionType)
+        return self._wrap(type(self.obj) is LambdaType)
 
     def isGenerator(self):
-        """
-        Check if the given object is GeneratorType
+        """ Check if the given object is GeneratorType
         """
         return self._wrap(type(self.obj) is GeneratorType)
 
     def isCode(self):
-        """
-        Check if the given object is CodeType
+        """ Check if the given object is CodeType
         """
         return self._wrap(type(self.obj) is CodeType)
 
     def isClass(self):
-        """
-        Check if the given object is ClassType
+        """ Check if the given object is ClassType
         """
         return self._wrap(type(self.obj) is ClassType)
 
     def isInstance(self):
-        """
-        Check if the given object is InstanceType
+        """ Check if the given object is InstanceType
         """
         return self._wrap(type(self.obj) is InstanceType)
 
     def isMethod(self):
-        """
-        Check if the given object is MethodType
+        """ Check if the given object is MethodType
         """
         return self._wrap(type(self.obj) is MethodType)
 
     def isUnboundMethod(self):
-        """
-        Check if the given object is UnboundMethodType
+        """ Check if the given object is UnboundMethodType
         """
         return self._wrap(type(self.obj) is UnboundMethodType)
 
     def isBuiltinFunction(self):
-        """
-        Check if the given object is BuiltinFunctionType
+        """ Check if the given object is BuiltinFunctionType
         """
         return self._wrap(type(self.obj) is BuiltinFunctionType)
 
     def isBuiltinMethod(self):
-        """
-        Check if the given object is BuiltinMethodType
+        """ Check if the given object is BuiltinMethodType
         """
         return self._wrap(type(self.obj) is BuiltinMethodType)
 
     def isModule(self):
-        """
-        Check if the given object is ModuleType
+        """ Check if the given object is ModuleType
         """
         return self._wrap(type(self.obj) is ModuleType)
 
     def isFile(self):
-        """
-        Check if the given object is FileType
+        """ Check if the given object is FileType
         """
         return self._wrap(type(self.obj) is FileType)
 
     def isXRange(self):
-        """
-        Check if the given object is XRangeType
+        """ Check if the given object is XRangeType
         """
         return self._wrap(type(self.obj) is XRangeType)
 
     def isSlice(self):
-        """
-        Check if the given object is SliceType
+        """ Check if the given object is SliceType
         """
         return self._wrap(type(self.obj) is SliceType)
 
     def isEllipsis(self):
-        """
-        Check if the given object is EllipsisType
+        """ Check if the given object is EllipsisType
         """
         return self._wrap(type(self.obj) is EllipsisType)
 
     def isTraceback(self):
-        """
-        Check if the given object is TracebackType
+        """ Check if the given object is TracebackType
         """
         return self._wrap(type(self.obj) is TracebackType)
 
     def isFrame(self):
-        """
-        Check if the given object is FrameType
+        """ Check if the given object is FrameType
         """
         return self._wrap(type(self.obj) is FrameType)
 
     def isBuffer(self):
-        """
-        Check if the given object is BufferType
+        """ Check if the given object is BufferType
         """
         return self._wrap(type(self.obj) is BufferType)
 
     def isDictProxy(self):
-        """
-        Check if the given object is DictProxyType
+        """ Check if the given object is DictProxyType
         """
         return self._wrap(type(self.obj) is DictProxyType)
 
     def isNotImplemented(self):
-        """
-        Check if the given object is NotImplementedType
+        """ Check if the given object is NotImplementedType
         """
         return self._wrap(type(self.obj) is NotImplementedType)
 
     def isGetSetDescriptor(self):
-        """
-        Check if the given object is GetSetDescriptorType
+        """ Check if the given object is GetSetDescriptorType
         """
         return self._wrap(type(self.obj) is GetSetDescriptorType)
 
     def isMemberDescriptor(self):
-        """
-        Check if the given object is MemberDescriptorType
+        """ Check if the given object is MemberDescriptorType
         """
         return self._wrap(type(self.obj) is MemberDescriptorType)
 
     def has(self, key):
         """
-        Shortcut function for checking if an object has a given property directly
-        on itself (in other words, not on a prototype).
+        Shortcut function for checking if an object has a
+        given property directly on itself (in other words, not on a prototype).
         """
         return self._wrap(hasattr(self.obj, key))
 
     def join(self, glue=" "):
-        """
-        Javascript's join implementation
+        """ Javascript's join implementation
         """
         j = glue.join([str(x) for x in self.obj])
         return self._wrap(j)
 
     def identity(self, *args):
-        """
-        Keep the identity function around for default iterators.
+        """ Keep the identity function around for default iterators.
         """
         return self._wrap(self.obj)
 
     def times(self, func, *args):
-        """
-        Run a function **n** times.
+        """ Run a function **n** times.
         """
         n = self.obj
         i = 0
@@ -1216,6 +1253,15 @@ class underscore():
 
         return self._wrap(func)
 
+    def random(self, max_number=None):
+        """ Return a random integer between min and max (inclusive).
+        """
+        min_number = self.obj
+        if max_number is None:
+            min_number = 0
+            max_number = self.obj
+        return random.randrange(min_number, max_number)
+
     def result(self, property, *args):
         """
         If the value of the named property is a function then invoke it;
@@ -1265,8 +1311,7 @@ class underscore():
     }
 
     def escape(self):
-        """
-        Escape a string for HTML interpolation.
+        """ Escape a string for HTML interpolation.
         """
         # & must be handled first
         self.obj = self.obj.replace("&", self._html_escape_table["&"])
@@ -1319,10 +1364,10 @@ class underscore():
     def template(self, data=None, settings=None):
         """
         Python micro-templating, similar to John Resig's implementation.
-        Underscore templating handles arbitrary delimiters, preserves whitespace,
-        and correctly escapes quotes within interpolated code.
+        Underscore templating handles arbitrary delimiters, preserves
+        whitespace, and correctly escapes quotes within interpolated code.
         """
-        if settings == None:
+        if settings is None:
             settings = {}
         ts = _.templateSettings
         _.defaults(ts, self.templateSettings)
@@ -1383,27 +1428,32 @@ class underscore():
             if code.startswith("end"):
                 return "')\n" + indent(-1) + "ns.__p += ('"
             elif code.endswith(':'):
-                return "')\n" + indent() + unescape(code) + "\n" + indent(+1) + "ns.__p += ('"
+                return "')\n" + indent() + unescape(code) + \
+                       "\n" + indent(+1) + "ns.__p += ('"
             else:
-                return "')\n" + indent() + unescape(code) + "\n" + indent() + "ns.__p += ('"
+                return "')\n" + indent() + unescape(code) + \
+                       "\n" + indent() + "ns.__p += ('"
 
         def escape(matchobj):
             key = (matchobj.group(1).decode('string-escape')).strip()
             return "' + _.escape(str(" + unescape(key) + " or '')) + '"
 
-        source = indent() + 'class closure:\n    pass # for full closure support\n'
+        source = indent() + 'class closure(object):\n    pass' + \
+                            ' # for full closure support\n'
         source += indent() + 'ns = closure()\n'
         source += indent() + "ns.__p = ''\n"
         #src = re.sub("^[\'\"]|[\'\"]$", "", ("%r" % src))
         src = re.sub(settings.get("escaper"), escapes, src)
-        source += indent() + "ns.__p += ('" + re.sub(settings.get('escape'), escape, src) + "')\n"
+        source += indent() + "ns.__p += ('" + \
+            re.sub(settings.get('escape'), escape, src) + "')\n"
         source = re.sub(settings.get('interpolate'), interpolate, source)
         source = re.sub(settings.get('evaluate'), evaluate, source)
         source += indent() + 'return ns.__p.decode("string_escape")\n'
 
-        f = self.create_function(settings.get("variable") or "obj=None", source)
+        f = self.create_function(settings.get("variable")
+                                 or "obj=None", source)
 
-        if data != None:
+        if data is not None:
             return f(data)
         return f
 
@@ -1415,7 +1465,7 @@ class underscore():
             exec code in globals(), locals()
         except:
             print source
-            raise  Exception("template error")
+            raise Exception("template error")
         ns.func = func
 
         def _wrap(obj={"this": ""}):
@@ -1427,15 +1477,13 @@ class underscore():
         return _wrap
 
     def chain(self):
-        """
-        Add a "chain" function, which will delegate to the wrapper.
+        """ Add a "chain" function, which will delegate to the wrapper.
         """
         self.chained = True
         return self
 
     def value(self):
-        """
-        returns the object instead of instance
+        """ returns the object instead of instance
         """
         if self._wrapped is not self.Null:
             return self._wrapped
@@ -1444,10 +1492,10 @@ class underscore():
 
     @staticmethod
     def makeStatic():
+        """ Provide static access to underscore class
         """
-        Provide static access to underscore class
-        """
-        for eachMethod in inspect.getmembers(underscore, predicate=inspect.ismethod):
+        for eachMethod in inspect.getmembers(underscore,
+                                             predicate=inspect.ismethod):
             m = eachMethod[0]
             if not hasattr(_, m):
                 def caller(a):
